/*
 *  GOL - Game of life simulation software
 *  Copyright (C) 1996 Michael Raymond Brown
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Michael Raymond Brown
 *  EMail:  michael@arrakeen.demon.co.uk
 *  WWW:    http://www.arrakeen.demon.co.uk/
 *  Address:  Michael Brown
 *            Kia Ora
 *            Brook Street
 *            Great Bromley
 *            Nr Colchester
 *            Essex
 *            CO7 7HX
 *            England
 */




#ifndef _tApp_h
#define _tApp_h

#ifdef __GNUC__
#pragma interface
#endif


#include "Core.h"

#include <Xm/Xm.h>


class xwApp : public xwCore
   {private:
	friend class xwImage;
#ifdef HAVE_MITSHM
	int	   noshm;
#endif
	XtAppContext app;
	Widget     shell;
	Widget   mainwin;

	Display* display;
	Screen *  screen;

	struct colcell
	   {int color;
	    int pixel;
	    int  next;
	   };

	union
	   {
	    struct
	       {unsigned int   red_mask,   red_up;
		unsigned int green_mask, green_up;
		unsigned int  blue_mask,  blue_up;
	       } xwTrueColor;
	    struct
	       {colcell* table;
		int      tsize;
		int*    lookup;
	       } xwPseudoColor;
	   } d;

	static int get_Black(xwApp*, int, int, int);
	static void destroy_Black(xwApp*);

	static int get_TrueColor(xwApp*, int, int, int);
	static class xwImage* image_TrueColor(xwApp*, int w, int h);

	static int get_PseudoColor(xwApp*, int, int, int);
	static void destroy_PseudoColor(xwApp*);
	static class xwImage* image_PseudoColor(xwApp*, int w, int h);

	void (*destroy)(xwApp*);
	int (*fget_rgb)(xwApp*, int, int, int);
	class xwImage* (*fget_image)(xwApp*, int, int);

	void setup_rgb();

    public:
	xwApp();
	xwApp(const char* app_name, int& argc, char** argv);
	virtual ~xwApp();

	void start();

	inline Display* getDisplay()
	   {return display; }
	inline Screen* getScreen()
	   {return screen ; }

	inline XtAppContext getApp()
	   {return app; }
	inline Widget getWindow()
	   {return mainwin; }
	inline Widget getShell()
	   {return shell; }

	inline int get_rgb(int r, int g, int b)
	   {return fget_rgb(this, r, g, b); }
#define get_gray	get_grey
	inline int get_grey(int grey)
	   {return fget_rgb(this, grey, grey, grey); }

	inline class xwImage* getImage(int w, int h)
	   {return fget_image(this, w, h); }

	inline int screenWidth()
	   {return WidthOfScreen(screen); }
	inline int screenHeight()
	   {return HeightOfScreen(screen); }
   };

#endif

